#include <iostream>
#include <list>
#include <string>
#include <algorithm>

#include "arccli.h"

#include <arc/jobftpcontrol.h>
#include <arc/joblist.h>
#include <arc/mdsquery.h>
#include <arc/notify.h>
#include <arc/resource.h>

#ifdef HAVE_LIBINTL_H
#include <libintl.h>
#define _(A) dgettext("arccli", (A))
#else
#define _(A) (A)
#endif


void arckill (const std::list<std::string> & jobs,
              const std::list<std::string> & clusterselect,
              const std::list<std::string> & clusterreject,
              const std::list<std::string> & status,
              const bool keep,
              const int timeout,
              const bool anonymous) {

	std::list<std::string> jobids =
		GetJobIDsList(jobs, clusterselect, clusterreject);

	if (jobs.empty() && clusterselect.empty() && jobids.empty())
		throw ARCCLIError(_("No jobs"));

	if (jobids.empty())
		throw ARCCLIError(_("No valid jobnames/jobids given"));

	std::list<Job> joblist =
		GetJobInfo(jobids, MDS_FILTER_JOBINFO, anonymous, "", timeout);

    for (std::list<Job>::iterator jli = joblist.begin();
         jli != joblist.end(); jli++) {
       if (jli->status.empty()) {
            try {
                Job job = GetJobInfoDirect(jli->id);
                *jli = job;
            } catch (ARCLibError) { }
        }
    }

	JobFTPControl ctrl;

	for (std::list<Job>::iterator jli = joblist.begin();
	     jli != joblist.end(); jli++) {

		if (jli->status.empty()) {
			notify(WARNING) << _("Job information not found")
			                << ": " << jli->id << std::endl;
			continue;
		}

		if (!status.empty() &&
		    std::find(status.begin(), status.end(), jli->status) == status.end())
			continue;

		if (jli->status == "DELETED") {
			notify(WARNING) << _("Job has already been deleted")
			                << ": " << jli->id << std::endl;
			continue;
		}

		if (jli->status == "FINISHED" || jli->status == "FAILED" ||
		    jli->status == "KILLED") {
			notify(WARNING) << _("Job has already finished")
			                << ": " << jli->id << std::endl;
			continue;
		}

		notify(WARNING) << _("Killing job")
		                << ": " << jli->id << std::endl;

		ctrl.Cancel(jli->id, timeout);

		if (!keep) {
			notify(WARNING) << _("Deleting job")
			                << ": " << jli->id << std::endl;

			ctrl.Clean(jli->id, timeout);
			RemoveJobID(jli->id);
		}
	}
}
